import uuid
from decimal import Decimal
from ool import VersionField, VersionedMixin
from django.core.validators import MinValueValidator
from django.db import models


class TicketBase(VersionedMixin, models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    title = models.CharField('Title', max_length=50)
    price = models.DecimalField('Price', max_digits=10, decimal_places=4,
                                validators=[MinValueValidator(Decimal('0.01'))])
    version = VersionField()

    class Meta:
        abstract = True


class OrderBase(VersionedMixin, models.Model):
    CREATED = 'created'
    CANCELLED = 'cancelled'
    AWAITINGPAYMENT = 'awaiting_payment'
    COMPLETE = 'complete'
    STATUS_CHOICES = [
        (CREATED, 'created'),
        (CANCELLED, 'cancelled'),
        (AWAITINGPAYMENT, 'awaiting_payment'),
        (COMPLETE, 'complete')
    ]

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    user_id = models.CharField('User ID', max_length=36)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default=CREATED)
    version = VersionField()

    class Meta:
        abstract = True



import boto3
import json
from abc import ABC, abstractclassmethod
from django.conf import settings


class PublisherBase(ABC):

    @abstractclassmethod
    def publish(cls, topic_name, body, delay):
        pass


class Sqs(PublisherBase):

    @classmethod
    def publish(cls, topic_name, body, delay):
        sns_client_options = {
            'region_name': settings.AWS_REGION_NAME,
            'aws_access_key_id': settings.AWS_ACCESS_KEY_ID,
            'aws_secret_access_key': settings.AWS_SECRET_ACCESS_KEY,
            'endpoint_url': settings.AWS_SNS_ENDPOINT_URL
        }
        sns_client = boto3.client("sns", **sns_client_options)
        sns_client.publish(
            TopicArn=settings.AWS_TOPIC_ARN_BASE + topic_name,
            Subject=topic_name,
            Message=json.dumps(body)
        )


class Publisher:
    SQS = 'SQS'
    RABBITMQ = 'RABBITMQ'

    @staticmethod
    def publish(topic_name, body, delay=None):
        publishers = {
            Publisher.SQS: Sqs()
        }
        publisher = publishers[settings.BROKER_NAME]
        publisher.publish(topic_name, body, delay)
